%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab routine for finding the direct parent for a given term in a given heirarchy
%%%        
%%% Inputs:   go          := a given term, value being the index of the term in go_list, scalar
%%%           root        := root of the hierarchy where go is from, value being the index of the root term in go_list, scalar
%%%           terms       := GO terms in the hierarchy, entries being indices of terms in go_by_go, 
%%%                          first entry is the root, (# terms) x 1
%%%           hie         := hierarchical structure matrix for terms, (# terms) x (# terms)
%%%           go_by_go    := go hierarchical structure matrix, entry (i, j) = 1 indicating that term j is an 
%%%                          ancestor of term j, 1927 x 1927
%%% Outputs:  y           := direct parent term in the given hierarchy, value being the index of the parent in go_list, scalar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = direct_pa( go, root, terms, hie, go_by_go )

desc = terms(2 : length(terms));
go_id = find(terms == go);

% Find all candidate parent terms of term go
cand_pa_id = find(hie(go_id, :) == 1);
sub_hie = hie([cand_pa_id, go_id], [cand_pa_id, go_id]);

for i = 1 : length(cand_pa_id)
    % If the number of descendants of candidate parent term i is greater
    % than 1, it is NOT the direct parent term 
    if length( find(sub_hie(:, i) == 1) ) > 1
        cand_pa_id(i) = 0;
    end;
end;
pa_id = cand_pa_id(find(cand_pa_id > 0));
y = terms(pa_id);
