%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% MatLab routine for Hierarchical Binomial-Neighborhood Algorithm on a set of genes and GO terms from a hierarchy
%%%
%%% Inputs:   genes       := genes to predict for, entries being indices of genes in gene_list, (# genes) x 1
%%%           terms       := GO terms in the hierarchy to predict for, entries being indices of terms 
%%%                          in go_by_go, first entry is the hierarchy root, (# terms) x 1
%%%           train       := genes used for training with entries being indices in ppi, (# train) x 1
%%%           ppi         := protein-protein interaction, 5143 x 5143
%%%           gene_by_go  := gene-GO association matrix, 5143 x 1927
%%%           go_by_go    := go hierarchical structure matrix, entry (i, j) = 1 indicating that term j is an 
%%%                          ancestor of term j, 1927 x 1927
%%% Outputs:  f           := probabilities of labels of terms on genes, (# genes) x (# terms)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = hbn_prob( genes, terms, train, ppi, gene_by_go, go_by_go )

root = terms(1);
desc = terms(2: length(terms));
% Transform DAG into tree
tree = mst(root, train, terms, gene_by_go, go_by_go);

for i = 1 : length(genes)
    for j = 2 : length(terms)
        
        ance = terms(find(tree(j, :) == 1));
        list = zeros(1, length(ance)+1);
        list(1) = terms(j);
        for t = 2 : length(find(tree(j, :) == 1))
            % Find the direct parent terms for terms along the path
            list(t) = direct_pa(list(t-1), root, terms, tree, go_by_go);
        end;
        list(length(ance) + 1) = root;
        p = zeros(length(ance), 1);

        for t = 1 : length(ance)
            p(t) = hbn_prob_one( list(t), list(t + 1), genes(i), train, ppi, gene_by_go );
        end;
        % Compute P(gene i has term j) = \prod P(gene i has )
        f(i, j-1) = prod(p);
    end;
end;