%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab routine for Minimal Spanning Tree Algorithm on GO hierarchy
%%%        
%%% Inputs:   root        := root of the hierarchy, value being the index of the root term in go_list, scalar
%%%           genes       := genes on which MST is performed, entries being indices of genes in gene_list, (# genes) x 1
%%%           terms       := GO terms in the hierarchy, entries being indices of terms in go_by_go, 
%%%                          first entry is the root, (# terms) x 1
%%%           gene_by_go  := gene-GO association matrix, 5143 x 1927
%%%           go_by_go    := go hierarchical structure matrix, entry (i, j) = 1 indicating that term j is an 
%%%                          ancestor of term j, 1927 x 1927
%%% Outputs:  y           := tree structure for terms, entry (i, j) = 1 indicating that term j is an ancestor of term j, 
%%%                          (# terms) x (# terms)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = mst( root, genes, terms, gene_by_go, go_by_go )

desc = terms(2 : length(terms));
% Structure matrix for terms
hie = go_by_go(terms, terms); 
n = length(terms);

% From the 2nd level (level directly below the root) and lower
leaf = terms(2 : n);
leaf(find(sum(hie') == 1)-1) = zeros(1, length(find(sum(hie') == 1)));
leaf = leaf(find(leaf > 0));

% If no 2nd level terms, keep the current hierarchy; o.w., 
if length(leaf) == 0
    y = hie;
else
    for i = 1 : length(leaf)
        if length(direct_pa(leaf(i), root, terms, hie, go_by_go)) == 1
            continue
        else
            
            leaf_id = find(terms == leaf(i));
            parents = direct_pa(leaf(i), root, terms, hie, go_by_go);
            p = zeros(1, length(parents));
            
            for j = 1 : length(parents)
                       
                a = gene_by_go( genes, leaf(i) ); 
                b = gene_by_go( genes, parents(j) );
            
                if length(find(a + b == 2)) == 0 && length(find(a + b == 0)) == 0
                    p(j) = 0;
                else
                    p(j) = length(find(a + b == 2))/(length(find(a + b == 2)) + length(find(a + b == 0)));
                end;
            end;
        
            pa = parents(find(p == max(p)));
            if length(pa) > 1
                pa = pa( randsample(1:length(pa), 1) );
            end;
            pa_id = find(terms == pa);
        
            % Relabel the entries in hie
            hie(leaf_id, :) = zeros(1, n);
            ance_id1 = [find(hie(pa_id, :) == 1), pa_id];
            hie(leaf_id, ance_id1) = ones(1, length(ance_id1));
            ch_id = find(hie(:, leaf_id) == 1);
            if length(ch_id) == 0
                continue
            else
                hie(ch_id, :) = zeros(length(ch_id), n);
                ance_id2 = [find(hie(leaf_id, :) == 1), leaf_id];
                hie(ch_id, ance_id2) = ones(length(ch_id), length(ance_id2));
            end;
         end;
    end;
    y = hie;
end;

     