%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab routine for Nearest Neighbor Algorithm
%%%  
%%% Inputs:   genes       := genes to predict for, entries being indices of genes in gene_list, (# genes) x 1
%%%           terms       := GO terms in the hierarchy to predict for, entries being indices of terms in go_by_go, 
%%%                          first entry is the root, (# terms) x 1
%%%           train       := genes used for training with entries being indices in ppi, (# train) x 1
%%%           ppi         := protein-protein interaction, 5143 x 5143
%%%           gene_by_go  := gene-GO association matrix, 5143 x 1927
%%% Outputs:  f           := probabilities of labels of terms on genes, (# genes) x (# terms)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = nn_prob( genes, terms, train, ppi, gene_by_go )

for i = 1 : length(genes)
    for j = 2 : length(terms)
        
        % In training set, find the neighbors of gene i
        neighbor = train(find(ppi(train, genes(i)) == 1));
        x = gene_by_go(:, terms(j));

        if length(neighbor) > 0
           f(i, j-1) = length(find(x(neighbor) == 1))/ length(neighbor);
        else
            % If gene i has no neighbor in the training set, use the
            % relative frequence of term j in the training set
           f(i, j-1) = length(find(gene_by_go(train, terms(j)) == 1))/length(train);
        end;

    end;
end;
