%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab Routine to generate ROC curves
%%%
%%%   Inputs:
%%%           thresholdold:= thresholdold applied to f, f >= thresholdold, prediction = 1; ow, prediction = -1, vector
%%%           Fold        := number of folds in cross validation, scalar 
%%%           terms       := GO terms from a given hierarchy that are prediected for, entries being indices of 
%%%                          the terms in go_list, vector
%%%           f           := probabilities from a certain method, (# terms) x (# genes predicted as defined in the routine)
%%%           ppi         := protein-protein interaction, 5143 x 5143
%%%           gene_by_go  := gene-GO association matrix, 5143 x 1927
%%%   Outputs:
%%%           plot of ROC curves based on all genes and all GO terms predicted
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function roc_plot(threshold, Fold, terms, f, ppi, gene_by_go)

root = terms(1);
desc = terms(2 : length(terms))';
genes = find(gene_by_go(:, root) == 1);
true = gene_by_go(genes, desc);

who = find(gene_by_go(:, root) == 1);

for fold = 1 : Fold
    if fold < Fold
        id = 1 + (fold - 1) * floor(length(sum(ppi))/Fold) : fold * floor(length(sum(ppi))/Fold);
        test = id(find(gene_by_go(id, root) == 1));
        n(fold) = length(test);
    elseif fold == Fold
        id = 1 + (fold - 1) * floor(length(sum(ppi))/Fold) : length(sum(ppi));
        test = id(find(gene_by_go(id, root) == 1));
        n(fold) = length(test);
    end;
end;

for i = 1 : length(threshold)
    for j = 1 : length(desc)
        true = gene_by_go(who, desc(j));
        prediction = f(j, :);
       
        [tp(j ,1), fp(j, 1), tn(j, 1), fn(j, 1)] = count(threshold(i), prediction(1 : n(1)), true(1: n(1)));
        for s = 2 : Fold
            [tp(j ,s), fp(j, s), tn(j, s), fn(j, s)] = count(threshold(i), prediction(sum(n(1:s-1))+1 : sum(n(1:s))), true(sum(n(1:s-1))+1 : sum(n(1:s))));
        end;
    end;
    allTP(i, :) = sum(tp);
    allFP(i, :) = sum(fp);
    allTN(i, :) = sum(tn);
    allFN(i, :) = sum(fn);

    for s = 1 : Fold
        send(i, s) = sum(allTP(i, s) + allFN(i, s));
        sped(i, s) = sum(allTN(i, s) + allFP(i, s));
        
        if sum(allTP(i, s) + allFN(i, s)) == 0
            sen(i, s) = 0; 
        else
            sen(i, s) = (allTP(i, s)./(allTP(i, s) + allFN(i, s)));
        end;
        
        if sum(allTN(i, s) + allFP(i, s)) == 0
            spe(i, s) = 0;
        else
            spe(i, s) = (allTN(i, s)./(allTN(i, s) + allFP(i, s)));
        end;
    end;
    d(i) = length(find(send(i, :) > 0));
    t(i) = length(find(sped(i, :) > 0));
end;

hold on;
axis([0, 1, 0, 1])
xlabel('1 - Specificity', 'FontSize', 15)
ylabel('Sensitivity', 'FontSize', 15)
plot(1 - sum(spe')./t, sum(sen')./d, '-r*', 'MarkerSize', 5) 
hold off;