%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab Routine to propagate a prediction following the true-path rule
%%%
%%%   Inputs:
%%%           prediction  := predicted labels from a certain method, (# terms) x (# genes predicted)
%%%           terms       := GO terms from a given hierarchy that are predicted for, entries being indices of 
%%%                          the terms in go_list, vector
%%%           go_by_go    := go hierarchical structure matrix, entry (i, j) = 1 indicating that term j is an 
%%%                          ancestor of term j, 1927 x 1927
%%%   Outputs:
%%%           y           := predicted labels after propagation, (# terms) x (# genes predicted)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = propagation(prediction, terms, go_by_go)

root = terms(1);
desc = terms(2 : length(terms));
hie = go_by_go(terms, terms);
[m, n] = size(prediction);

for i = 1 : n
    list = find(prediction(:, i) == 1);
    for j = 1 : length(list)
        ance = find(hie(list(j) + 1, :) == 1);
        if length(ance) == 0
            continue
        else 
            for k = 1 : length(ance)
                if ance(k) == 1
                    continue
                else
                    if prediction(ance(k) - 1, i) == -1
                        prediction(ance(k) - 1, i) = 1;
                    end;
                end;
            end;
        end;
    end;
    
end;
y = prediction;